//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"
#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Button::Button(int xt, int yt, int w, int h, int rt, int gt, int bt, string s, float name_vert_frac)
{
  x = xt;
  y = yt;

  width = w;
  height = h;

  r = rt;
  g = gt;
  b = bt;

  name = s;

  name_vert_fraction = name_vert_frac;

  //  active = true;
  active = false;

  name_string = (char *) malloc(MAX_STRLEN*sizeof(char));

  num_sprites = 0;
}

//----------------------------------------------------------------------------

void Button::add_sprite(Sprite *spr, double s, double t, double sdelta, double tdelta, float sprite_vert_frac)
{
  if (num_sprites >= MAX_BUTTON_SPRITES)
    return;

  sprite_vert_fraction = sprite_vert_frac;

  sprite[num_sprites] = spr;
  sprite_s[num_sprites] = s;
  sprite_t[num_sprites] = t;
  sprite_sdelta[num_sprites] = sdelta;
  sprite_tdelta[num_sprites] = tdelta;

  num_sprites++;
}

//----------------------------------------------------------------------------

void Button::draw()
{
  glColor3ub(r, g, b);
  
  glBegin(GL_POLYGON);
  
  glVertex2f(x - 0.5 * width, y - 0.5 * height);
  glVertex2f(x - 0.5 * width, y + 0.5 * height);
  glVertex2f(x + 0.5 * width, y + 0.5 * height);
  glVertex2f(x + 0.5 * width, y - 0.5 * height);
  
  glEnd();
  
  // draw a row of all added sprites below button name

  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);

  float y_sprite = y + height * (0.5 - sprite_vert_fraction);

  //  printf("%f %f %f %f\n", sprite_s[0], sprite_t[0], sprite_sdelta[0], sprite_tdelta[0]);

  for (int i = 0; i < num_sprites; i++) {

    sprite[i]->draw_sub_sprite(x + ((float) i - 0.5 * (float)(num_sprites-1)) * sprite_sdelta[i]*(double)sprite[i]->imwidth, y_sprite, 
			       sprite_sdelta[i]*(double)sprite[i]->imwidth, sprite_tdelta[i]*(double)sprite[i]->imheight, 
			       0.0,
			       sprite_s[i], sprite_t[i], sprite_sdelta[i], sprite_tdelta[i],
			       1, 1, 1);
    //exit(1);
    /*
    sprite[i]->s = sprite_s[i];
    sprite[i]->t = sprite_t[i];
    sprite[i]->sdelta = sprite_sdelta[i];
    sprite[i]->tdelta = sprite_tdelta[i];
    sprite[i]->draw(x + ((float) i - 0.5 * (float)(num_sprites-1)) * sprite_sdelta[i]*(double)sprite[i]->imwidth, y_sprite, 
		    sprite_sdelta[i]*(double)sprite[i]->imwidth, sprite_tdelta[i]*(double)sprite[i]->imheight, 0.0, 1, 1, 1);
    sprite[i]->s = sprite[i]->t = 0.0; 
    sprite[i]->sdelta = sprite[i]->tdelta = 1.0;
    */
  }


  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);

  // draw name

  sprintf(name_string, "%s", name.c_str());
  //  draw_centered_string(x, y, BUTTON_CHAR_WIDTH, name_string, 0, 0, 0);
  //  draw_centered_bitmap_string(x, y, name_string, GLUT_BITMAP_TIMES_ROMAN_24, 0, 0, 0);
  
  float y_name = y + height * (0.5 - name_vert_fraction);
  //  float y_name = y + 0.5 * height - name_vert_fraction * height

  if (active) {
    draw_centered_bitmap_string(x, y_name, name_string, GLUT_BITMAP_HELVETICA_18, 0, 0, 0);

    // a little hack here to distinguish resource buttons from action buttons

    if (strlen(name_string) == 0) {

      glColor3ub(255, 0, 255);
      glLineWidth(4);
      
      glBegin(GL_LINE_LOOP);
      
      glVertex2f(x - 0.5 * width, y - 0.5 * height);
      glVertex2f(x - 0.5 * width, y + 0.5 * height);
      glVertex2f(x + 0.5 * width, y + 0.5 * height);
      glVertex2f(x + 0.5 * width, y - 0.5 * height);
      
      glEnd();

    }
  }
  else
    draw_centered_bitmap_string(x, y_name, name_string, GLUT_BITMAP_HELVETICA_18, 128, 128, 128);


  // restore pen state
  
  glColor3ub(255,255,255);  
  glLineWidth(1);
}

//----------------------------------------------------------------------------

void Button::set_inactive()
{
  active = false;
}

//----------------------------------------------------------------------------

void Button::set_active()
{
  active = true;
}

//----------------------------------------------------------------------------
 
bool Button::is_clicked(int mouse_x, int mouse_y)
{
  if (!active)
    return false;

  if (mouse_x >= x - width/2 && mouse_x < x + width/2 && 
      mouse_y >= y - height/2 && mouse_y < y + height/2)
    return true;
  else
    return false;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

