//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"
#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Catan::Catan(int np)
{
  int i, j;

  init_random();

  // this places terrain and number tokens

  board = new Board();   

  // initialize player objects

  num_players = np;
  player = new Player* [num_players];
  for (i = 0; i < num_players; i++)
    player[i] = new Player(i);

  // clear turn counter

  state_string = new char[MAX_STRLEN];
  resource_string = new char[MAX_STRLEN];
  victory_string = new char[MAX_STRLEN];
  dev_card_string = new char[MAX_STRLEN];

  phase = SETUP_PHASE;
  turn_number = 0;
  which_player = 0;
  player_delta = +1;
  current_piece = PIECE_SETTLEMENT;
  trade_giving = true;

  cursor_x = cursor_y = UNKNOWN_CURSOR_POSITION;

  // development cards

  //  num_development_cards = NUM_TOTAL_CARDS;
  dev_card_deck.clear();
  for (i = 0; i < NUM_CARD_TYPES; i++)
    for (j = 0; j < NUM_CARDS[i]; j++) 
      dev_card_deck.push_back(i);

  // shuffle
  int temp;
  for (i = 0; i < dev_card_deck.size() - 1; i++) {
    j = uniform_random_int(i + 1, dev_card_deck.size() - 1);
    temp = dev_card_deck[i];
    dev_card_deck[i] = dev_card_deck[j];
    dev_card_deck[j] = temp; 
  }

//    for (i = 0; i < dev_card_deck.size(); i++) 
//      printf("%i %i\n", i, dev_card_deck[i]);
// 
//   printf("num dev cards = %i\n", dev_card_deck.size());
//   exit(1);
  //    development_card[i] = NUM_CARDS[i];
  dev_card_monopoly = false;
  dev_card_year_of_plenty = 0;

  // load sprites

  read_sprites();

  // initialize buttons

  setup_buttons();
}

//----------------------------------------------------------------------------

// "draw" a card from the deck

int Catan::random_development_card()
{
  // none left

  if (dev_card_deck.size() == 0)
    return CARD_NONE;

  // at least one...

  int which_card = dev_card_deck[dev_card_deck.size() - 1];
  dev_card_deck.pop_back();

  return which_card;
}

//----------------------------------------------------------------------------

// draw here means "render"

// current_card hold info in which one has been drawn

void Catan::draw_development_card(int x, int y, int w, int h)
{
  // fill

  glColor3ub(OFFWHITE_R, OFFWHITE_G, OFFWHITE_B);
  
  glBegin(GL_POLYGON);
  
  glVertex2f(x - 0.5 * w, y - 0.5 * h);
  glVertex2f(x - 0.5 * w, y + 0.5 * h);
  glVertex2f(x + 0.5 * w, y + 0.5 * h);
  glVertex2f(x + 0.5 * w, y - 0.5 * h);
  
  glEnd();

  // outline

  glColor3ub(PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  glLineWidth(2);
  
  glBegin(GL_LINE_LOOP);
  
  glVertex2f(x - 0.5 * w, y - 0.5 * h);
  glVertex2f(x - 0.5 * w, y + 0.5 * h);
  glVertex2f(x + 0.5 * w, y + 0.5 * h);
  glVertex2f(x + 0.5 * w, y - 0.5 * h);
  
  glEnd();

  // strings

  if (CARD_DESCRIPTION2[player[which_player]->current_card].length() == 0) {

    sprintf(dev_card_string, "%s", CARD_NAME[player[which_player]->current_card].c_str());
    draw_centered_bitmap_string(x, y + 0.2*h, dev_card_string, GLUT_BITMAP_HELVETICA_18, 
				PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  
    sprintf(dev_card_string, "%s", CARD_DESCRIPTION1[player[which_player]->current_card].c_str());
    draw_centered_bitmap_string(x, y - 0.1*h, dev_card_string, GLUT_BITMAP_HELVETICA_12, 
				PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  }
  else {

    sprintf(dev_card_string, "%s", CARD_NAME[player[which_player]->current_card].c_str());
    draw_centered_bitmap_string(x, y + 0.3*h, dev_card_string, GLUT_BITMAP_HELVETICA_18, 
				PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);

    sprintf(dev_card_string, "%s", CARD_DESCRIPTION1[player[which_player]->current_card].c_str());
    draw_centered_bitmap_string(x, y, dev_card_string, GLUT_BITMAP_HELVETICA_12, 
				PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);

    sprintf(dev_card_string, "%s", CARD_DESCRIPTION2[player[which_player]->current_card].c_str());
    draw_centered_bitmap_string(x, y - 0.2*h, dev_card_string, GLUT_BITMAP_HELVETICA_12, 
				PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  }
}

//----------------------------------------------------------------------------

// attempt to put a building down at intersection nearest to window coordinate

// return true if success, fail otherwise (so click can be ignored)

// if check_road true, require that building be "on a road" belonging to
// current player (as opposed to isolated)

bool Catan::try_to_place_building(int type, int x, int y, bool check_road)
{
  int inter_row, inter_col;
  int i;
  bool road_is_good;
  Intersection *I;

  I = board->get_closest_intersection(x, y);

  if (!I)
    return false;

  // check that all adjacent intersections are empty

  for (inter_row = 0; inter_row < MAX_VERTICAL_INTERSECTIONS; inter_row++) 
    for (inter_col = 0; inter_col < MAX_HORIZONTAL_INTERSECTIONS; inter_col++) 
      if (I->is_adjacent(board->intersection[inter_row][inter_col]) && 
	  board->intersection[inter_row][inter_col]->building != PIECE_NONE)
	return false;

  // make sure we're on a road

  if (check_road) {

    // consider all edges which include this intersection...do any of them have
    // a road owned by this player?

    road_is_good = false;
    for (i = 0; i < board->edge_list.size(); i++)
      if (board->edge_adjacent_to_intersection(board->edge_list[i], I) &&
	  board->edge_list[i]->road && board->edge_list[i]->owned_by == which_player) {
	road_is_good = true;
	break;
      }

    if (!road_is_good)
      return false;
  }

  // now make sure everything is good _at_ the intersection

  if ((type == PIECE_SETTLEMENT && I->building == PIECE_NONE) || 
      (type == PIECE_CITY && I->building == PIECE_SETTLEMENT && I->owned_by == which_player)) {

    I->building = type;                  // there's a building there now
    I->owned_by = which_player;          // and it's owned by this player

    if (type == PIECE_SETTLEMENT) {
      player[which_player]->num_settlements++;
      player[which_player]->adjust_trade_ratios_if_port(I->port);  // if this is a port, trading ratios may need changing
    }
    else if (type == PIECE_CITY) {
      player[which_player]->num_settlements--;
      player[which_player]->num_cities++;
    }

    // placement successful...

    // ...if it's setup phase, switch to road piece.  if it's this player's
    // last turn, distribute resources

    if (phase == SETUP_PHASE) {

      current_piece = PIECE_ROAD;

      if (is_final_turn_of_setup_phase())
	distribute_setup_phase_production(I);

    }

    // ...else assume it's turn phase; we're holding one less piece 

    else if (--num_current_pieces == 0)
      end_piece_placement_mode();

    return true;

  }

  // bad news: things are not good at the intersection :(

  else
    return false;
}

//----------------------------------------------------------------------------

// attempt to put a road down at edge nearest to window coordinate

// return true if success, fail otherwise (so click can be ignored)

// if check_road true, require that this road be connected to another
// of same player

bool Catan::try_to_place_road(int x, int y, int constraint)
{
  bool road_is_good;
  int i;
  Edge *E;
  Intersection *I;

  E = board->get_closest_edge(x, y);

  if (!E)
    return false;

  // check unoccupied

  if (E->road)
    return false;

  // make sure we're extending an existing road

  if (constraint == EXTENDS_ROAD) {

    road_is_good = false;

    // consider all edges which touch this edge...do any of them have
    // a road owned by this player?

    for (i = 0; i < board->edge_list.size(); i++)
      if (board->edge_adjacent_to_edge(board->edge_list[i], E) &&
	  board->edge_list[i]->road && board->edge_list[i]->owned_by == which_player) {
	road_is_good = true;
	break;
      }

    if (!road_is_good)
      return false;
  }

  // make sure we're touching a settlement owned by the same player that no other road is touching
  // (setup phase only)

  else if (constraint == TOUCHES_ONLY_SETTLEMENT) {

    // touches a settlement?

    if (E->I1->building == PIECE_SETTLEMENT)
      I = E->I1;
    else if (E->I2->building == PIECE_SETTLEMENT)
      I = E->I2;
    else
      return false;

    // settlement owned by same player?

    if (I->owned_by != which_player)
      return false;

    // any other road touching settlement?

    for (i = 0; i < board->edge_list.size(); i++)
      if (board->edge_adjacent_to_intersection(board->edge_list[i], I) && board->edge_list[i]->road)
	return false;
  }

  // good to go...

  E->road = true;
  E->owned_by = which_player;

  // if it's setup phase, switch to settlement piece

  if (phase == SETUP_PHASE) {

    current_piece = PIECE_SETTLEMENT;

  }

  // else assume it's turn phase; we're holding one less piece
  
  else if (--num_current_pieces == 0)
    end_piece_placement_mode();

  return true;
}

//----------------------------------------------------------------------------

// check_different_hex true means robber can't be dropped where it started

bool Catan::try_to_place_robber(int x, int y, bool check_different_hex)
{
  int hex_row, hex_col;

  if (!board->window_point_inside_which_hex(x, y, hex_row, hex_col))
    return false;

  board->hex[hex_row][hex_col]->robber = true;

  end_piece_placement_mode();

  return true;
}

//----------------------------------------------------------------------------

// must read sprites first

void Catan::setup_buttons()
{
  // player choice buttons

  roll_dice_button      = new Button(BUTTON_X, BUTTON_Y, 
				     BUTTON_WIDTH, BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "Roll dice");
  end_turn_button       = new Button(BUTTON_X, BUTTON_Y - 1*(BUTTON_HEIGHT + BUTTON_SPACE), 
				     BUTTON_WIDTH, BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "End turn");
  trade_button          = new Button(BUTTON_X, BUTTON_Y - 2*(BUTTON_HEIGHT + BUTTON_SPACE), 
				     BUTTON_WIDTH, BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "Trade resources");

  buy_road_button       = new Button(BUTTON_X, BUTTON_Y - 2*(BUTTON_HEIGHT + BUTTON_SPACE)-0.5*BUTTON_HEIGHT-BUTTON_SPACE-0.5*BUY_BUTTON_HEIGHT, 
				     BUTTON_WIDTH, BUY_BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "Buy road", 0.25);
  buy_road_button->add_sprite(resource_sprite[RESOURCE_BRICK], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_road_button->add_sprite(resource_sprite[RESOURCE_LUMBER], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);

  buy_settlement_button = new Button(BUTTON_X, BUTTON_Y - 2*(BUTTON_HEIGHT + BUTTON_SPACE)-0.5*BUTTON_HEIGHT-2*BUTTON_SPACE-1.5*BUY_BUTTON_HEIGHT, 
				     BUTTON_WIDTH, BUY_BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "Buy settlement", 0.25);
  buy_settlement_button->add_sprite(resource_sprite[RESOURCE_BRICK], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_settlement_button->add_sprite(resource_sprite[RESOURCE_LUMBER], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_settlement_button->add_sprite(resource_sprite[RESOURCE_WOOL], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_settlement_button->add_sprite(resource_sprite[RESOURCE_GRAIN], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);

  buy_city_button       = new Button(BUTTON_X, BUTTON_Y - 2*(BUTTON_HEIGHT + BUTTON_SPACE)-0.5*BUTTON_HEIGHT-3*BUTTON_SPACE-2.5*BUY_BUTTON_HEIGHT,
				     BUTTON_WIDTH, BUY_BUTTON_HEIGHT, 
				     BUTTON_R, BUTTON_G, BUTTON_B, "Buy city", 0.25);
  buy_city_button->add_sprite(resource_sprite[RESOURCE_ORE], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_city_button->add_sprite(resource_sprite[RESOURCE_ORE], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_city_button->add_sprite(resource_sprite[RESOURCE_ORE], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_city_button->add_sprite(resource_sprite[RESOURCE_GRAIN], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_city_button->add_sprite(resource_sprite[RESOURCE_GRAIN], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);

  buy_development_card_button = new Button(BUTTON_X, BUTTON_Y - 2*(BUTTON_HEIGHT + BUTTON_SPACE)-0.5*BUTTON_HEIGHT-4*BUTTON_SPACE-3.5*BUY_BUTTON_HEIGHT,
					   BUTTON_WIDTH, BUY_BUTTON_HEIGHT, 
					   BUTTON_R, BUTTON_G, BUTTON_B, "Buy development card", 0.25);
  buy_development_card_button->add_sprite(resource_sprite[RESOURCE_ORE], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_development_card_button->add_sprite(resource_sprite[RESOURCE_WOOL], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);
  buy_development_card_button->add_sprite(resource_sprite[RESOURCE_GRAIN], RESOURCE_SUB_S, RESOURCE_SUB_T, RESOURCE_SUB_SDELTA, RESOURCE_SUB_TDELTA);

  // resource card buttons

  for (int i = 0; i < NUM_RESOURCES; i++) {
    resource_button[i] = new Button(CARD_X_OFFSET + WINDOW_WIDTH/2 + (i-2)*(CARD_WIDTH + CARD_SPACING), CARD_Y, CARD_WIDTH, CARD_HEIGHT, 255, 255, 255, "");
    resource_button[i]->add_sprite(resource_sprite[i], 0.0, 0.0, 1.0, 1.0, 0.5);
  }
}

//----------------------------------------------------------------------------

void Catan::deactivate_action_buttons()
{
  trade_button->set_inactive();
  roll_dice_button->set_inactive();
  end_turn_button->set_inactive();
  buy_road_button->set_inactive();
  buy_settlement_button->set_inactive();
  buy_city_button->set_inactive();
  buy_development_card_button->set_inactive();
}

//----------------------------------------------------------------------------

void Catan::deal_and_play_development_card()
{
  int i;

  player[which_player]->current_card = random_development_card();

  // automatic victory point

  if (player[which_player]->current_card >= CARD_PALACE && player[which_player]->current_card <= CARD_LIBRARY) 
    player[which_player]->dev_card_victory_points++;

  // road building

  else if (player[which_player]->current_card == CARD_ROAD_BUILDING) 
    start_piece_placement_mode(PIECE_ROAD, 2);

  // monopoly

  else if (player[which_player]->current_card == CARD_MONOPOLY) {
    dev_card_monopoly = true;
    for (i = 0; i < NUM_RESOURCES; i++) 
      resource_button[i]->set_active();
  }

  // year of plenty

  else if (player[which_player]->current_card == CARD_YEAR_OF_PLENTY) {
    dev_card_year_of_plenty = 2;
    for (i = 0; i < NUM_RESOURCES; i++) 
      resource_button[i]->set_active();
  }

  // knight

  else if (player[which_player]->current_card == CARD_KNIGHT) {
    move_robber();
  }

}

//----------------------------------------------------------------------------

// resource button should have been clicked.  was one?  if so, which one?

void Catan::handle_resource_buttons(int x, int y)
{
  int i;
  int resource_chosen = RESOURCE_NOTHING;

  // which resource card was chosen?

  for (i = 0; i < NUM_RESOURCES; i++)
    if (resource_button[i]->is_clicked(x, y)) {
      resource_chosen = i;
      break;
    }

  if (resource_chosen == RESOURCE_NOTHING)
    return;

  // clicked because player drew monopoly development card:
  // go through all of other players and subtract chosen resource from them, add to this player

  if (dev_card_monopoly) {

    for (i = 0; i < num_players; i++)
      if (i != which_player) {
	player[which_player]->resources[resource_chosen] += player[i]->resources[resource_chosen];
	player[i]->resources[resource_chosen] = 0;
      }

    dev_card_monopoly = false;
    player[which_player]->current_card = CARD_NONE;
    for (i = 0; i < NUM_RESOURCES; i++) 
      resource_button[i]->set_inactive();
  }

  // clicked because player drew year of plenty development card:

  else if (dev_card_year_of_plenty) {

    player[which_player]->resources[resource_chosen]++;

    if (--dev_card_year_of_plenty == 0) {
      player[which_player]->current_card = CARD_NONE;
      for (i = 0; i < NUM_RESOURCES; i++) 
	resource_button[i]->set_inactive();
    }
  }

  // clicked because player is trading (with the bank)

  // first click: which resource is player GIVING in this trade?

  else if (trade_giving) {
    //    player[which_player]->resources[resource_chosen] -= TRADE_RATIO_ANYTIME;
    player[which_player]->resources[resource_chosen] -= player[which_player]->trade_ratio[resource_chosen];
    trade_giving = false;
    for (i = 0; i < NUM_RESOURCES; i++) {
      if (i == resource_chosen)
	resource_button[i]->set_inactive();
      else
	resource_button[i]->set_active();
    }
  }

  // second click: which resource is player GETTING in this trade?

  else {
    player[which_player]->resources[resource_chosen] += 1;
    trade_giving = true;
    for (i = 0; i < NUM_RESOURCES; i++) 
      resource_button[i]->set_inactive();

    set_buy_button_activations();
    set_trade_button_activation();
    roll_dice_button->set_inactive();
    end_turn_button->set_active();
  }
}

//----------------------------------------------------------------------------

// return any highlighted hexes or buttons to their normal appearances

void Catan::clear_highlights()
{
  board->clear_hex_highlights();
  clear_button_highlights();
}

//----------------------------------------------------------------------------

void Catan::clear_button_highlights()
{
  int i;

  for (int i = 0; i < NUM_RESOURCES; i++)
    resource_button[i]->set_inactive();
}

//----------------------------------------------------------------------------

 // read in sprite source images

void Catan::read_sprites()
{
  RGB white = {255, 255, 255};
  RGB green = {0, 255, 0};
  RGB almost_green = {1, 255, 1};
  RGB blue = {0, 0, 255};
  RGB none = {-1, -1, -1};

  clear_sprites();                                            // call this once, _before_ adding any info

  add_sprite_info("images/card_brick.ppm", TEXTURE_BRICK, none);
  add_sprite_info("images/card_wool.ppm", TEXTURE_WOOL, none);  
  add_sprite_info("images/card_ore.ppm", TEXTURE_ORE, none);    
  add_sprite_info("images/card_grain.ppm", TEXTURE_GRAIN, none);
  add_sprite_info("images/card_lumber.ppm", TEXTURE_LUMBER, none);  

  add_sprite_info("images/board.ppm", TEXTURE_BOARD, white);

  add_sprite_info("images/tile_hills.ppm", TEXTURE_HILLS, blue);
  add_sprite_info("images/tile_pasture.ppm", TEXTURE_PASTURE, blue);
  add_sprite_info("images/tile_mountains.ppm", TEXTURE_MOUNTAINS, blue);
  add_sprite_info("images/tile_fields.ppm", TEXTURE_FIELDS, blue);
  add_sprite_info("images/tile_forest.ppm", TEXTURE_FOREST, blue);
  add_sprite_info("images/tile_desert.ppm", TEXTURE_DESERT, blue);

  add_sprite_info("images/trade_wildcard.ppm", TEXTURE_WILDCARD, none);
  add_sprite_info("images/robber.ppm", TEXTURE_ROBBER, green);

  finalize_sprites();                                         // call this once, _after_ adding all info

  // make actual sprites from source images

  resource_sprite[RESOURCE_BRICK] = new Sprite(TEXTURE_BRICK);
  resource_sprite[RESOURCE_WOOL] = new Sprite(TEXTURE_WOOL);
  resource_sprite[RESOURCE_ORE] = new Sprite(TEXTURE_ORE);
  resource_sprite[RESOURCE_GRAIN] = new Sprite(TEXTURE_GRAIN);
  resource_sprite[RESOURCE_LUMBER] = new Sprite(TEXTURE_LUMBER);

  board_background_sprite = new Sprite(TEXTURE_BOARD);

  terrain_sprite[TERRAIN_HILLS] = new Sprite(TEXTURE_HILLS);
  terrain_sprite[TERRAIN_PASTURE] = new Sprite(TEXTURE_PASTURE);
  terrain_sprite[TERRAIN_MOUNTAINS] = new Sprite(TEXTURE_MOUNTAINS);
  terrain_sprite[TERRAIN_FIELDS] = new Sprite(TEXTURE_FIELDS);
  terrain_sprite[TERRAIN_FOREST] = new Sprite(TEXTURE_FOREST);
  terrain_sprite[TERRAIN_DESERT] = new Sprite(TEXTURE_DESERT);

  trade_wildcard_sprite = new Sprite(TEXTURE_WILDCARD);
  robber_sprite = new Sprite(TEXTURE_ROBBER);

}

//----------------------------------------------------------------------------

// should be called right after constructor

void finalize(Catan *C)
{
  int row, col;
  int i;

  C->board->game = C;

  // hexes link to game object

  for (row = 0; row < MAX_HEXES_PER_SIDE; row++) 
    for (col = 0; col < MAX_HEXES_PER_SIDE; col++)
      if (C->board->hex[row][col])
	C->board->hex[row][col]->game = C;

  // edges link to board object
  
  for (i = 0; i < C->board->edge_list.size(); i++)
    C->board->edge_list[i]->board = C->board;
}

//----------------------------------------------------------------------------

// are we on second pick (as opposed to first pick) of setup phase?

bool Catan::is_final_turn_of_setup_phase()
{
  return (phase == SETUP_PHASE && player_delta < 0);
}

//----------------------------------------------------------------------------

// one player just finished turn--now what?

void Catan::next_player()
{
  if (phase == SETUP_PHASE) {

    // if that was the last player, "turn around" (future updates are decrements)

    if (which_player == num_players - 1 && player_delta == +1) 
      player_delta = -1;

    // else just update player (+1 or -1) 

    else {

      which_player += player_delta;

      // if this player update takes us into negative territory, the setup phase is done

      if (which_player < 0) 
	start_turn_phase();
    }
  }

  else if (phase == TURN_PHASE) {

    // if that was the last player, it's a new turn -- go back to the beginning

    if (which_player == num_players - 1) {
      which_player = 0;
      turn_number++;
      roll_dice_button->set_active();
      end_turn_button->set_inactive();
    }
    else {
      which_player++;
      roll_dice_button->set_active();
      end_turn_button->set_inactive();
    }

    //    player[which_player]->current_card = CARD_NONE;

  }

  else {
    cout << "There should not be a next player if it's not the setup or turn phase\n";
    exit(1);
  }

    /*
  // if that was the last player, it's a new turn

  if (which_player == num_players - 1 && player_delta == +1) {

    // go back to beginning

    if (phase == TURN_PHASE) {
      which_player = 0;
      turn_number++;
    }

    // turn around

    else if (phase == SETUP_PHASE) 
      player_delta = -1;

  }

  // else just increment player (+1 or -1) --same turn

  else {

    which_player += player_delta;

    if (phase == TURN_PHASE) {
      roll_dice_button->set_active();
      end_turn_button->set_inactive();
    }

    // this should only happen at end of setup phase

    else if (phase == SETUP_PHASE && which_player < 0) 
      start_turn_phase();
  }
    */
}

//----------------------------------------------------------------------------

void Catan::start_turn_phase()
{
  phase = TURN_PHASE;
  which_player = 0;
  current_piece = PIECE_NONE;

  roll_dice_button->set_active();
}

//----------------------------------------------------------------------------

// print message string to window

void Catan::draw_state()
{
  if (phase == SETUP_PHASE) {
    if (current_piece == PIECE_SETTLEMENT)
      sprintf(state_string, "Setup: Pick a settlement");
    else
      sprintf(state_string, "Setup: Pick a road");
    draw_centered_bitmap_string(STATE_X, STATE_LINE1_Y, state_string, GLUT_BITMAP_HELVETICA_18, 0, 0, 0);
    sprintf(state_string, "%s player's turn", PLAYER_COLOR[which_player].c_str());
    draw_centered_bitmap_string(STATE_X, STATE_LINE2_Y, state_string, GLUT_BITMAP_HELVETICA_18, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  }
  else if (phase == TURN_PHASE) {
    sprintf(state_string, "Turn %i", turn_number);
    draw_centered_bitmap_string(STATE_X, STATE_LINE1_Y, state_string, GLUT_BITMAP_HELVETICA_18, 0, 0, 0);
    sprintf(state_string, "%s player's turn", PLAYER_COLOR[which_player].c_str());
    draw_centered_bitmap_string(STATE_X, STATE_LINE2_Y, state_string, GLUT_BITMAP_HELVETICA_18, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]
);
  }
  else if (phase == GAMEOVER_PHASE) {
    sprintf(state_string, "Game over");
    draw_centered_bitmap_string(STATE_X, STATE_LINE1_Y, state_string, GLUT_BITMAP_HELVETICA_18, 0, 0, 0);
    sprintf(state_string, "%s player won!", PLAYER_COLOR[winning_player].c_str());
    draw_centered_bitmap_string(STATE_X, STATE_LINE2_Y, state_string, GLUT_BITMAP_HELVETICA_18, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]
);
  }
}

//----------------------------------------------------------------------------

// if player is placing something, draw it under cursor

void Catan::draw_cursor()
{
  if (cursor_x == UNKNOWN_CURSOR_POSITION || cursor_y == UNKNOWN_CURSOR_POSITION)
    return;

  if (current_piece == PIECE_ROBBER)
    draw_robber(cursor_x, cursor_y, robber_sprite);
  else if (current_piece == PIECE_ROAD)
    draw_road(cursor_x, cursor_y, 8, 0.9*(float)HEX_GRID_SIDE, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  else if (current_piece == PIECE_SETTLEMENT)
    draw_settlement(cursor_x, cursor_y, SETTLEMENT_WIDTH, SETTLEMENT_HEIGHT, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  else if (current_piece == PIECE_CITY)
    draw_city(cursor_x, cursor_y, PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
}


//----------------------------------------------------------------------------

void Catan::draw_action_buttons()
{
  roll_dice_button->draw();
  buy_road_button->draw();
  buy_settlement_button->draw();
  buy_city_button->draw();
  buy_development_card_button->draw();
  trade_button->draw();
  end_turn_button->draw();
}

//----------------------------------------------------------------------------

// how many of each card does the current player have?

void Catan::draw_player_resources()
{
  // draw resource cards -- same for every player -- and overlay trade ratio

  for (int i = 0; i < NUM_RESOURCES; i++) {

    // button itself

    resource_button[i]->draw();

    // trade ratio info

    draw_filled_circle(resource_button[i]->x + resource_button[i]->width/4, resource_button[i]->y + resource_button[i]->height/3, 10, OFFWHITE_R, OFFWHITE_G, OFFWHITE_B);
    sprintf(resource_string, "x%i", player[which_player]->trade_ratio[i]);
    draw_centered_bitmap_string(resource_button[i]->x + resource_button[i]->width/4, resource_button[i]->y + resource_button[i]->height/3, resource_string, GLUT_BITMAP_HELVETICA_12, 0, 0, 0);

  }

  // draw how much of each resource player has below corresponding card

  for (int i = 0; i < NUM_RESOURCES; i++) {   
    sprintf(resource_string, "%i", player[which_player]->resources[i]);
    draw_centered_bitmap_string(CARD_X_OFFSET + WINDOW_WIDTH/2 + (i - 2)*(CARD_WIDTH + CARD_SPACING), RESOURCE_Y, resource_string, GLUT_BITMAP_HELVETICA_18, 0, 0, 0);
  }

  // current development card

  if (player[which_player]->current_card != CARD_NONE) {
//     printf("which player %i\n", which_player);
//     fflush(stdout);
//     printf("current card %i\n", player[which_player]->current_card);
//     fflush(stdout);

    draw_development_card(DEV_CARD_X, DEV_CARD_Y, DEV_CARD_W, DEV_CARD_H);

  }

}

//----------------------------------------------------------------------------

// how many victory points does current player have?

void Catan::draw_player_victory_points()
{
  // compute

  player[which_player]->compute_victory_points();

  if (phase == TURN_PHASE && player[which_player]->victory_points >= WIN_VICTORY_POINTS) {
    phase = GAMEOVER_PHASE;
    winning_player = which_player;
    deactivate_action_buttons();
    glutPostRedisplay();
  }

  // fill

  glColor3ub(OFFWHITE_R, OFFWHITE_G, OFFWHITE_B);
  glLineWidth(2);
  
  glBegin(GL_POLYGON);
  
  glVertex2f(VICTORY_PTS_X - 0.5 * CARD_HEIGHT, VICTORY_PTS_Y - 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X - 0.5 * CARD_HEIGHT, VICTORY_PTS_Y + 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X + 0.5 * CARD_HEIGHT, VICTORY_PTS_Y + 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X + 0.5 * CARD_HEIGHT, VICTORY_PTS_Y - 0.5 * CARD_HEIGHT);
  
  glEnd();

  // outline

  glColor3ub(PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);
  glLineWidth(2);
  
  glBegin(GL_LINE_LOOP);
  
  glVertex2f(VICTORY_PTS_X - 0.5 * CARD_HEIGHT, VICTORY_PTS_Y - 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X - 0.5 * CARD_HEIGHT, VICTORY_PTS_Y + 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X + 0.5 * CARD_HEIGHT, VICTORY_PTS_Y + 0.5 * CARD_HEIGHT);
  glVertex2f(VICTORY_PTS_X + 0.5 * CARD_HEIGHT, VICTORY_PTS_Y - 0.5 * CARD_HEIGHT);
  
  glEnd();

  // strings

  sprintf(victory_string, "%i", player[which_player]->victory_points);
  draw_centered_bitmap_string(VICTORY_PTS_X, VICTORY_PTS_Y + 10, victory_string, GLUT_BITMAP_TIMES_ROMAN_24,
			      PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);

  sprintf(victory_string, "VP");
  draw_centered_bitmap_string(VICTORY_PTS_X, VICTORY_PTS_Y - 15, victory_string, GLUT_BITMAP_HELVETICA_18,
			      PLAYER_R[which_player], PLAYER_G[which_player], PLAYER_B[which_player]);

}

//----------------------------------------------------------------------------

// deduct cost of resources from current player and update button 
// activations which depend on how many resources player has

void Catan::execute_player_purchase(const int *price)
{
  // do the subtraction

  for (int i = 0; i < NUM_RESOURCES; i++)
    player[which_player]->resources[i] -= price[i];

  // since player has fewer resources now...

  update_resource_sensitive_buttons();
}

//----------------------------------------------------------------------------

void Catan::update_resource_sensitive_buttons()
{
  // what can we buy now?

  set_buy_button_activations();

  // can we trade now?

  set_trade_button_activation();
}

//----------------------------------------------------------------------------

void Catan::start_piece_placement_mode(int which_piece, int how_many)
{
  deactivate_action_buttons();
  current_piece = which_piece;
  num_current_pieces = how_many;
}

//----------------------------------------------------------------------------

void Catan::end_piece_placement_mode()
{
  player[which_player]->current_card = CARD_NONE;

  current_piece = PIECE_NONE;
  update_resource_sensitive_buttons();

  roll_dice_button->set_inactive();
  end_turn_button->set_active();

  //  deactivate_action_buttons();
}

//----------------------------------------------------------------------------

// set proper active status for road/settlement/city/development card buttons based on 
// resources of current player

// city should only be activated if player has at least 1 settlement

void Catan::set_buy_button_activations()
{
  int i;

  // road

  buy_road_button->set_active();
  for (i = 0; i < NUM_RESOURCES; i++)
    if (player[which_player]->resources[i] < COST_ROAD[i]) {
      buy_road_button->set_inactive();
      break;
    }

  // settlement

  buy_settlement_button->set_active();
  for (i = 0; i < NUM_RESOURCES; i++)
    if (player[which_player]->resources[i] < COST_SETTLEMENT[i]) {
      buy_settlement_button->set_inactive();
      break;
    }

  // city

  buy_city_button->set_active();
  for (i = 0; i < NUM_RESOURCES; i++)
    if (player[which_player]->resources[i] < COST_CITY[i]) {
      buy_city_button->set_inactive();
      break;
    }

  // development card

  //  if (num_development_cards > 0) {
  if (dev_card_deck.size() > 0) {

    buy_development_card_button->set_active();

    for (i = 0; i < NUM_RESOURCES; i++)
      if (player[which_player]->resources[i] < COST_DEVELOPMENT_CARD[i]) {
	buy_development_card_button->set_inactive();
	break;
      }
  }
}


//----------------------------------------------------------------------------

// set proper status for trade button based on resources of current player.
// if a player can trade ANY resource, this button should be active

void Catan::set_trade_button_activation()
{
  int i;

  trade_button->set_inactive();
  for (i = 0; i < NUM_RESOURCES; i++)
    //    if (player[which_player]->resources[i] >= TRADE_RATIO_ANYTIME) {
    if (player[which_player]->resources[i] >= player[which_player]->trade_ratio[i]) {
      trade_button->set_active();
      break;
    }
}

//----------------------------------------------------------------------------

// set proper status for trade button based on resources of current player,
// taking port ownership into consideration

void Catan::set_resource_button_activations()
{
  for (int i = 0; i < NUM_RESOURCES; i++)
    //    if (player[which_player]->resources[i] >= TRADE_RATIO_ANYTIME) 
    if (player[which_player]->resources[i] >= player[which_player]->trade_ratio[i])
      resource_button[i]->set_active();
}

//----------------------------------------------------------------------------

// given a specific dice roll, look up which hexes have that token value
// and then give that hex's resource to all players with adjacent buildings

void Catan::distribute_production(int dice_total)
{
  int row, col;

  for (row = 0; row < MAX_HEXES_PER_SIDE; row++) 
    for (col = 0; col < MAX_HEXES_PER_SIDE; col++) 
      if (board->hex[row][col] && board->hex[row][col]->token_value == dice_total) {
	board->hex[row][col]->draw_highlighted = true;

	// does robber suppress production?

	if (!board->hex[row][col]->robber)
	  board->hex[row][col]->distribute_resource_to_adjacent_intersections();
      }

  // move robber on a 7, wherever he is

  if (dice_total == ROBBER_TOKEN) 
    move_robber();

}

//----------------------------------------------------------------------------

// find where robber is now, remove him, and get ready for current player to place him

void Catan::move_robber()
{
  int row, col;

  for (row = 0; row < MAX_HEXES_PER_SIDE; row++) 
    for (col = 0; col < MAX_HEXES_PER_SIDE; col++) 
      if (board->hex[row][col] && board->hex[row][col]->robber) {
	board->hex[row][col]->robber = false;
	start_piece_placement_mode(PIECE_ROBBER);
	break;
      }

}

//----------------------------------------------------------------------------

// given a specific intersection (and using which_player), give them resources from
// surrounding three hexes.  this is done for the second settlement picked in the
// setup phase

void Catan::distribute_setup_phase_production(Intersection *I)
{
  int row, col;
  int terrain;

  for (row = 0; row < MAX_HEXES_PER_SIDE; row++) 
    for (col = 0; col < MAX_HEXES_PER_SIDE; col++) 
      if (board->hex[row][col] && board->hex[row][col]->is_adjacent(I)) {
	terrain = board->hex[row][col]->terrain;
	if (terrain != TERRAIN_DESERT) {
	  //	  printf("distributing terrain %i to player %i\n", terrain, I->owned_by); 
	  player[I->owned_by]->resources[terrain] += RESOURCE_INCREMENT_SETTLEMENT;
	}
      }
}

//----------------------------------------------------------------------------

// "roll" two six-sided dice

int Catan::roll_dice()
{
  int die1, die2;

  die1 = uniform_random_int(1, 6);
  die2 = uniform_random_int(1, 6);

  return die1 + die2;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
