//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#ifndef CATAN_H
#define CATAN_H

//----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#include <iostream>
#include <cstdlib>
#include <string>
#include <vector> 

#include "Sprite.hh"

using namespace std;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

const int MAX_STRLEN           = 256;

const int NUM_TOTAL_HEXES      = 19;
const int MAX_HEXES_PER_SIDE   = 5;
 
const int MAX_VERTICAL_INTERSECTIONS   = 1 + MAX_HEXES_PER_SIDE;
const int MAX_HORIZONTAL_INTERSECTIONS = 2 * (1 + MAX_HEXES_PER_SIDE); // this # on even rows, 1 less on odd

const int NUM_TERRAINS         = 6;

const int TERRAIN_HILLS        = 0;
const int TERRAIN_PASTURE      = 1;
const int TERRAIN_MOUNTAINS    = 2;
const int TERRAIN_FIELDS       = 3;
const int TERRAIN_FOREST       = 4;
const int TERRAIN_DESERT       = 5;

// order here corresponds to numbering above

const char TERRAIN_CODE[NUM_TERRAINS] = {'h', 'p', 'm', 'f', 'F', 'd' };

const int NUM_HILLS_HEXES      = 3;
const int NUM_PASTURE_HEXES    = 4;
const int NUM_MOUNTAINS_HEXES  = 3;
const int NUM_FIELDS_HEXES     = 4;
const int NUM_FOREST_HEXES     = 4;
const int NUM_DESERT_HEXES     = 1;

const int NUM_TOKEN_VALUES     = 13;  // 0-12, inclusive
const int LOW_TOKEN_VALUE      = 2;
const int HIGH_TOKEN_VALUE     = 12;
const int ROBBER_TOKEN         = 7;
//const int NO_TOKEN             = 0;

// these should not be adjacent hexes

const int SPECIAL_TOKEN_VALUE1 = 6;
const int SPECIAL_TOKEN_VALUE2 = 8;

// use hex to print--so 10 = 'A', 11 = 'B', and 12 = 'C'

const int NUM_0s               = 0;
const int NUM_1s               = 0;
const int NUM_2s               = 1;
const int NUM_3s               = 2;
const int NUM_4s               = 2;
const int NUM_5s               = 2;
const int NUM_6s               = 2;
const int NUM_7s               = 0;  // there is one, but we say 0 here so it won't be chosen randomly
const int NUM_8s               = 2;
const int NUM_9s               = 2;
const int NUM_10s              = 2;
const int NUM_11s              = 2;
const int NUM_12s              = 1;

const int NUM_WILDCARD_PORTS   = 8;
const int NUM_RESOURCE_PORTS   = 2;  // per resource type

const int WILDCARD_PORT_ROW[NUM_WILDCARD_PORTS] = {  0,  0,  2,  3,  3,  4,  5,  5 };
const int WILDCARD_PORT_COL[NUM_WILDCARD_PORTS] = {  5,  6,  0,  1, 10, 11, 10, 11  }; 

const int BRICK_PORT_ROW[NUM_RESOURCE_PORTS]    = { 5, 5 };
const int BRICK_PORT_COL[NUM_RESOURCE_PORTS]    = { 7, 8 };

const int WOOL_PORT_ROW[NUM_RESOURCE_PORTS]     = { 1, 2 };
const int WOOL_PORT_COL[NUM_RESOURCE_PORTS]     = { 8, 9 };

const int ORE_PORT_ROW[NUM_RESOURCE_PORTS]      = { 0, 0 };
const int ORE_PORT_COL[NUM_RESOURCE_PORTS]      = { 2, 3 };

const int GRAIN_PORT_ROW[NUM_RESOURCE_PORTS]    = { 1, 1 };
const int GRAIN_PORT_COL[NUM_RESOURCE_PORTS]    = { 0, 1 };

const int LUMBER_PORT_ROW[NUM_RESOURCE_PORTS]   = { 4, 4 };
const int LUMBER_PORT_COL[NUM_RESOURCE_PORTS]   = { 3, 4 };

const int NUM_CARD_TYPES           = 9; // not including CARD_NONE here

const int CARD_PALACE              = 0; // +1 victory point (palace, university, market, chapel, library)
const int CARD_UNIVERSITY          = 1;
const int CARD_MARKET              = 2;
const int CARD_CHAPEL              = 3;
const int CARD_LIBRARY             = 4;
const int CARD_ROAD_BUILDING       = 5; // 2 free roads to build right away
const int CARD_YEAR_OF_PLENTY      = 6; // 2 free resources (different or the same) from the bank
const int CARD_MONOPOLY            = 7; // choose 1 resource; transfer all of that type from other players to you
const int CARD_KNIGHT              = 8; // move robber right away
const int CARD_NONE                = 9; // the "null" card

const int NUM_TOTAL_CARDS                     = 25;

const int NUM_CARDS[NUM_CARD_TYPES]           = { 1, 1, 1, 1, 1, 2, 2, 2, 14 };

const string CARD_NAME[NUM_CARD_TYPES]        = {"Palace", "University", "Market", "Chapel", "Library", 
						"Road Building", "Year of Plenty", "Monopoly", "Knight" };

const string CARD_DESCRIPTION1[NUM_CARD_TYPES] = { "1 Victory Point!", 
						   "1 Victory Point!", 
						   "1 Victory Point!", 
						   "1 Victory Point!", 
						   "1 Victory Point!",
						   "Place 2 roads for free",
						   "Take any 2 resources",
						   "Take all of one resource",
						   "Move the robber" };

const string CARD_DESCRIPTION2[NUM_CARD_TYPES] = { "", 
						   "", 
						   "", 
						   "", 
						   "",
						   "",
						   "from bank",
						   "from other players",
						   "" };

// convention is that intersection 0 is top, go in clockwise order

//       0
//       ^
//      / \
//  5  /   \ 1
//     |   |
//     |   |
//  4  \   / 2
//      \ /
//       v
//       3

const int NUM_INTERSECTIONS_PER_HEX  = 6;
const int MAX_HEXES_PER_INTERSECTION = 3;
const int MAX_EDGES_PER_INTERSECTION = 3;
const int NO_INTERSECTION            = -1;

const int PIECE_ROAD         = 1;
const int PIECE_SETTLEMENT   = 2;
const int PIECE_CITY         = 3;
const int PIECE_ROBBER       = 4;
const int PIECE_NONE         = 5;

//----------------------------------------------------------------------------

class Hex;
class Edge;
class Intersection;
class Board;
class Catan;

//----------------------------------------------------------------------------

class Hex 
{
public:
  Hex(int r, int c, Board *B) 
  { 
    row = r, col = c;
    board = B;
    draw_highlighted = false;
    robber = false;
    for (int i = 0; i < NUM_INTERSECTIONS_PER_HEX; i++)
      inter[i] = NULL;
  }

  bool window_point_inside(float, float);
  void compute_window_vertices(float, float, float);
  void draw_terrain();
  void draw_outline(int, int, int, int, float = 1.0);
  void draw_token_value();
  bool is_adjacent(Hex *);
  bool is_adjacent(Intersection *);
  int intersection_index(Intersection *);
  Intersection *find_intersection_with_index(int);
  void print_hex_adjacencies();
  void print_intersection_adjacencies();
  void distribute_resource_to_adjacent_intersections();
  bool is_special();
  bool is_adjacent_to_special();

  Board *board;       // parent
  Catan *game;

  int row, col;       // where located in board grid

  int terrain;        // terrain type for this hex
  int token_value;    // dice roll number which makes this hex produce
  bool robber;        // is robber on this hex?
 
  // intersections adjacent to this hex

  Intersection *inter[NUM_INTERSECTIONS_PER_HEX];

  // window coordinates of vertices of this hex

  float xc, yc;              // center of hex
  float x_window_diff[NUM_INTERSECTIONS_PER_HEX];   // x offset from xc (for drawing scaled version of hex)
  float y_window_diff[NUM_INTERSECTIONS_PER_HEX];   // x offset from xc (for drawing scaled version of hex)

  bool draw_highlighted;
};

//----------------------------------------------------------------------------

class Intersection
{
public:
  Intersection(int, int, Board *);

  void draw();
  void draw_highlighted(int, int, int, int);
  bool is_adjacent(Intersection *);

  Board *board;             // parent

  int row, col;             // where located in board grid

  float x_window, y_window; // window coordinates of this intersection

  int building;             // building type here (could be empty)
  int owned_by;             // if not empty, which player has it

  int port;                 // port type here (could be empty)

};

//----------------------------------------------------------------------------

class Edge
{
public:
  Edge(Intersection *inter1, Intersection *inter2) { road = false; I1 = inter1; I2 = inter2; }

  Board *board;       // parent

  bool road;          // is there a road here?
  int owned_by;       // if not empty, which player has it

  // intersections which define this edge

  Intersection *I1, *I2;

  // never need to know what hexes are adjacent to an edge

};

//----------------------------------------------------------------------------

class Board
{
public:
  Board();

  bool is_board_hex(int row, int col) { return abs(col - row) <= 2; }
  bool is_board_intersection(int row, int col) { return (col - 2*row) <= 6 && (2*row - col) <= 5; }

  int pick_random_available(int [], int);

  void initialize_hexes();
  void initialize_terrain_types();
  bool initialize_token_values();
  void initialize_intersections();
  void initialize_edges();

  void print_terrain_types(bool = false);
  void print_token_values();
  void print_intersections();
  void print_all_hex_adjacencies();
  void print_all_intersection_adjacencies();

  void compute_window_vertices(float, float, float);
  void draw();
  bool window_point_inside_which_hex(float, float, int &, int &);
  Intersection *get_closest_intersection(int, int);
  Edge *get_closest_edge(int, int);
  void clear_hex_highlights();
  bool is_new_edge(Intersection *, Intersection *);
  double point_edge_distance(double, double, double, double, double, double);
  bool edge_adjacent_to_intersection(Edge *, Intersection *);
  bool edge_adjacent_to_edge(Edge *, Edge *);

  Catan *game;

  // following this pattern:

  // 000xx
  // 0000x
  // 00000
  // x0000
  // xx000

  // some of grid, intersection, and edge arrays are NULL -- 
  // these are "unused" due to shape of board

  Hex *hex[MAX_HEXES_PER_SIDE][MAX_HEXES_PER_SIDE];

  // list of all intersections, each appearing only once

  Intersection *intersection[MAX_VERTICAL_INTERSECTIONS][MAX_HORIZONTAL_INTERSECTIONS];

  // list of all edges, each appearing only once

  //  Edge *edge[MAX_EDGES];
  vector <Edge *> edge_list;

  bool hex_highlighted;
  int highlight_row;
  int highlight_col;

  bool inter_highlighted;
  int highlight_inter_row;
  int highlight_inter_col;

};

//----------------------------------------------------------------------------

// 1-1 mapping between these and terrains

const int NUM_RESOURCES      = NUM_TERRAINS - 1;

const int RESOURCE_BRICK     = TERRAIN_HILLS;
const int RESOURCE_WOOL      = TERRAIN_PASTURE;
const int RESOURCE_ORE       = TERRAIN_MOUNTAINS;
const int RESOURCE_GRAIN     = TERRAIN_FIELDS;
const int RESOURCE_LUMBER    = TERRAIN_FOREST;

const int RESOURCE_NOTHING   = TERRAIN_DESERT;

const int RESOURCE_WILDCARD  = TERRAIN_DESERT + 1;   // used for ? ports

const int POINTS_SETTLEMENT   = 1;
const int POINTS_CITY         = 2;
const int POINTS_LONGEST_ROAD = 2;
const int POINTS_LARGEST_ARMY = 2;

const int MIN_ROADS_LONGEST_ROAD   = 5;
const int MIN_KNIGHTS_LARGEST_ARMY = 5;

const int WIN_VICTORY_POINTS  = 10;

const string RESOURCE_CODE[NUM_RESOURCES] = {"Brick", "Wool", "Ore", "Grain", "Lumber" };

const int RESOURCE_INCREMENT_SETTLEMENT  = 1;
const int RESOURCE_INCREMENT_CITY        = 2;

// price menu

const int COST_ROAD[NUM_RESOURCES]              = { 1, 0, 0, 0, 1 };
const int COST_SETTLEMENT[NUM_RESOURCES]        = { 1, 1, 0, 1, 1 };
const int COST_CITY[NUM_RESOURCES]              = { 0, 0, 3, 2, 0 };
const int COST_DEVELOPMENT_CARD[NUM_RESOURCES]  = { 0, 1, 1, 1, 0 };

// "special" road placement constraints (beyond not overwriting existing road, etc.)

const int EXTENDS_ROAD            = 1;
const int TOUCHES_ONLY_SETTLEMENT = 2;
const int NO_CONSTRAINT           = 3;

//----------------------------------------------------------------------------

class Player
{
public:
  Player(int);

  void print();
  void reset_trade_ratios(int, int);
  void adjust_trade_ratios_if_port(int);
  void compute_victory_points();
 
  int player_number;               // ID to tell players apart

  int resources[NUM_RESOURCES];    // how much "money" player has
  int trade_ratio[NUM_RESOURCES];  // what is trade ratio for each resource for this player

  int current_card;             

  int num_settlements;
  int num_cities;

  int num_roads;
  bool longest_road;

  int num_knights;
  bool largest_army;

  int dev_card_victory_points;

  int victory_points;
};

//----------------------------------------------------------------------------

const int MAX_BUTTON_SPRITES    = 5;

class Button
{
public:
  Button(int, int, int, int, int, int, int, string, float = 0.5);

  void draw();
  void set_inactive();
  void set_active();
  bool is_clicked(int, int);
  void add_sprite(Sprite *, double, double, double, double, float = 0.70);

  int x, y;
  int width, height;
  int r, g, b;
  string name;
  bool active;
  char *name_string;
  float name_vert_fraction;   // 0.5 means draw name vertically centered.  0.25 means closer to top, 0.75 means closer to bottom, etc.
  float sprite_vert_fraction; // same as above, but for all sprites added
  int num_sprites;
  Sprite *sprite[MAX_BUTTON_SPRITES];
  double sprite_s[MAX_BUTTON_SPRITES];
  double sprite_t[MAX_BUTTON_SPRITES];
  double sprite_sdelta[MAX_BUTTON_SPRITES];
  double sprite_tdelta[MAX_BUTTON_SPRITES];
};

//----------------------------------------------------------------------------

const int MAX_PLAYERS         = 4;

const string PLAYER_COLOR[MAX_PLAYERS] = {"Blue", "Red", "White", "Orange" };

const int PLAYER_R[MAX_PLAYERS] = {   0, 255, 255, 255 };
const int PLAYER_G[MAX_PLAYERS] = {   0,   0, 255, 165 };
const int PLAYER_B[MAX_PLAYERS] = { 255,   0, 255,   0 };

const int SPLASH_PHASE        = 0;
const int SETUP_PHASE         = 1;
const int TURN_PHASE          = 2;
const int GAMEOVER_PHASE      = 3;

const int TRADE_RATIO_ANYTIME       = 4;
const int TRADE_RATIO_WILDCARD_PORT = 3;
const int TRADE_RATIO_RESOURCE_PORT = 2;


class Catan
{
public:
  Catan(int);

  int roll_dice();      
  void distribute_production(int);
  void distribute_setup_phase_production(Intersection *);
  void draw_state();
  void next_player();
  void draw_player_resources();
  void draw_player_victory_points();
  void execute_player_purchase(const int *);
  void read_sprites();
  void draw_action_buttons();
  void draw_roll_dice_button();
  void start_turn_phase();
  void set_buy_button_activations();
  void set_trade_button_activation();
  void set_resource_button_activations();
  void draw_cursor();
  void setup_buttons();
  bool is_final_turn_of_setup_phase();
  void clear_highlights();
  void clear_button_highlights();
  void handle_resource_buttons(int, int);
  void deactivate_action_buttons();
  void start_piece_placement_mode(int, int = 1);
  void end_piece_placement_mode();
  void update_resource_sensitive_buttons();
  bool try_to_place_building(int, int, int, bool);
  bool try_to_place_road(int, int, int);
  bool try_to_place_robber(int, int, bool);
  int random_development_card();
  void draw_development_card(int, int, int, int);
  void deal_and_play_development_card();
  void move_robber();

  Board *board;  
  int num_players;
  Player **player;

  char *state_string;
  char *resource_string;
  char *victory_string;
  char *dev_card_string;

  int phase;

  int turn_number;
  int which_player;
  int player_delta;            // ordinarily +1, but is -1 for second half of setup phase
  int winning_player;          // set when end of game is detected

  int current_piece;           // what piece is "in hand", ready for placement (if any)?
  int num_current_pieces;      // usually 1, but can be more if multiple copies should be placed
  int cursor_x, cursor_y;      // where is cursor right now, whether clicked or not?

  //  int num_development_cards;   // how many are left in the deck
  //  int development_card[NUM_CARD_TYPES];    
  vector <int> dev_card_deck;
  bool dev_card_monopoly;      // are we executing a monopoly play?
  int dev_card_year_of_plenty; // how many year of plenty selections are left?
  
  bool trade_giving;    // true for the first resource chosen; false for the second (aka "trade_getting")

  // all buttons go here

  Button *roll_dice_button;
  Button *buy_road_button;
  Button *buy_settlement_button;
  Button *buy_city_button;
  Button *buy_development_card_button;
  Button *trade_button;
  Button *end_turn_button;

  Button *resource_button[NUM_RESOURCES];

  // all sprites go here

  Sprite *terrain_sprite[NUM_TERRAINS];
  Sprite *resource_sprite[NUM_RESOURCES];

  Sprite *trade_wildcard_sprite;
  Sprite *board_background_sprite;
  Sprite *robber_sprite;

};


//----------------------------------------------------------------------------

// created this because it I had some problems trying to do it in the constructor and gave up

void finalize(Catan *);

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#endif
