//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// utility functions (mostly drawing) for OpenGL-based Catan game
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void draw_robber(float x, float y, Sprite *S)
{
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);
  
  S->draw(x, y, 3*S->imwidth/5, 3*S->imheight/5, 0, 1, 1, 1); 
  
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
}

//----------------------------------------------------------------------------

void draw_road(float x, float y, float width, float height, int r, int g, int b)
{
  glColor3ub(r, g, b);
  
  glBegin(GL_POLYGON);
  
  glVertex2f(x - 0.5 * width, y - 0.5 * height);   // bottom left
  glVertex2f(x - 0.5 * width, y + 0.5 * height);
  glVertex2f(x + 0.5 * width, y + 0.5 * height);
  glVertex2f(x + 0.5 * width, y - 0.5 * height);
  
  glEnd();
  
  // restore pen state
  
  glColor3ub(255,255,255);  
}

//----------------------------------------------------------------------------

void draw_settlement(float x, float y, float width, float height, int r, int g, int b)
{
  glColor3ub(r, g, b);
  
  glBegin(GL_POLYGON);
  
  glVertex2f(x - 0.5 * width, y - 0.33 * height);   // bottom left
  glVertex2f(x - 0.5 * width, y + 0.33 * height);
  glVertex2f(x, y + 0.67 * height);                 // roof peak
  glVertex2f(x + 0.5 * width, y + 0.33 * height);
  glVertex2f(x + 0.5 * width, y - 0.33 * height);
  
  glEnd();

  glColor3ub(0, 0, 0);
  glLineWidth(1);

  glBegin(GL_LINE_LOOP);
  
  glVertex2f(x - 0.5 * width, y - 0.33 * height);   // bottom left
  glVertex2f(x - 0.5 * width, y + 0.33 * height);
  glVertex2f(x, y + 0.67 * height);                 // roof peak
  glVertex2f(x + 0.5 * width, y + 0.33 * height);
  glVertex2f(x + 0.5 * width, y - 0.33 * height);
  
  glEnd();
  
  // restore pen state
  
  glColor3ub(255,255,255);  
}

//----------------------------------------------------------------------------

void draw_city(float x, float y, int r, int g, int b)
{
  draw_settlement(x - 0.25 * SETTLEMENT_WIDTH, y, 0.5 * SETTLEMENT_WIDTH, SETTLEMENT_HEIGHT, r, g, b);
  draw_settlement(x + 0.25 * SETTLEMENT_WIDTH, y, 0.5 * SETTLEMENT_WIDTH, SETTLEMENT_HEIGHT, r, g, b);
}

//----------------------------------------------------------------------------

// sample shape-drawing function

void draw_line(float x1, float y1, float x2, float y2, int r, int g, int b, int thickness)
{
  // change pen color, size

  glColor3ub(r, g, b);
  glLineWidth(thickness);

  glBegin(GL_LINES);
    glVertex2f(x1, y1);
    glVertex2f(x2, y2);
  glEnd();

  // restore pen state

  glColor3f(1,1,1);
  glLineWidth(1);
}

//----------------------------------------------------------------------------

void draw_filled_circle(float x, float y, float radius, int r, int g, int b)
{
  double x1, y1;
  double theta, dtheta, step_size;
  int i;

  step_size = 0.1;
  dtheta = step_size / radius;

  glColor3ub(r, g, b);
  
  glBegin(GL_POLYGON);

  for (theta = 0.0 ; theta < 2*M_PI; theta += dtheta) {

    x1 = x + radius * cos(theta);
    y1 = y + radius * sin(theta);

    glVertex2f(x1, y1);
  }

  glEnd();

  glColor3f(1,1,1);

}

//----------------------------------------------------------------------------

void draw_centered_bitmap_string(float x, float y, char *text, void *font, int r, int g, int b)
{
  int len, i, bitmap_len, y_offset;

  bitmap_len = glutBitmapLength(font, (unsigned char *) text);

  if (font == GLUT_BITMAP_TIMES_ROMAN_24)
    y_offset = 9;
  else if (font == GLUT_BITMAP_HELVETICA_18)
    y_offset = 9;
  else if (font == GLUT_BITMAP_HELVETICA_12)
    y_offset = 6;
  else if (font == GLUT_BITMAP_TIMES_ROMAN_10 || font == GLUT_BITMAP_HELVETICA_10)
    y_offset = 5;
  else
    y_offset = 7;

  glColor3ub(r, g, b);
  glRasterPos2f(x - bitmap_len/2, y - y_offset);
  len = (int) strlen(text);
  for (i = 0; i < len; i++)
    glutBitmapCharacter(font, text[i]);

  glColor3ub(255, 255, 255);

}

//----------------------------------------------------------------------------

// MONO characters are all same width: 104.76 pixels

// x, y is center of string horizontally, bottom vertically

void draw_centered_string(float x, float y, float char_width, char *text, int r, int g, int b)
{
  char *p;
  int len = strlen(text);
  float x_offset = 0.5 * ((float) len) * char_width;
  float y_offset = 0.5 * char_width;

  glPushMatrix();
  glColor3ub(r, g, b);
  glTranslatef(x - x_offset, y - y_offset, 0);
  glScalef(char_width / 104.76, char_width / 104.76, 1);
  for (p = text; *p; p++) {

    // characters are all the same widths -- not so pretty, but easier to predict space taken
    
    glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, *p);
    //   glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
    
  }

  glPopMatrix();
}

//----------------------------------------------------------------------------

// MONO characters are all same width: 104.76 pixels

// x, y is lower-left corner of string

void draw_string(float x, float y, float char_width, char *text, int r, int g, int b, bool mono)
{
  char *p;
   
  glPushMatrix();
  glColor3ub(r, g, b);
  glTranslatef(x, y, 0);
  glScalef(char_width / 104.76, char_width / 104.76, 1);
  for (p = text; *p; p++) {

    // characters are all the same widths -- not so pretty, but easier to predict space taken
    
    if (mono)
      glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, *p);
    
    // proportional: looks better, but characters are different widths
    
    else
      glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
  }

  glPopMatrix();
}

//----------------------------------------------------------------------------

// set OpenGL states, variables

void init_opengl(string window_title, int window_w, int window_h) 
{
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowSize(window_w, window_h);
  glutCreateWindow(window_title.c_str());

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluOrtho2D(0, window_w, 0, window_h);

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  // background color (sea blue)

  //  glClearColor(SEA_R/255.0, SEA_G/255.0, SEA_B/255.0, 0.0);
  glClearColor(0.6, 0.6, 0.7, 0);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// set seed based on time so no two games are alike

void init_random()
{
  struct timeval tp;

  gettimeofday(&tp, NULL);
  srand48(tp.tv_sec);
}

//----------------------------------------------------------------------------

// random float in range [lower, upper]

double uniform_random(double lower, double upper)
{
  double result;
  double range_size;

  range_size = upper - lower;
  result = range_size * drand48();
  result += lower;

  return result;
}

//----------------------------------------------------------------------------

// random int in range [lower, upper]

int uniform_random_int(int lower, int upper)
{
  return (int) round(uniform_random(lower, upper));
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

