//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// utility functions for OpenGL-based Catan game
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#ifndef GAME_UTILS_H
#define GAME_UTILS_H

//----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <sys/time.h>

#include <iostream>
#include <cstdlib>
#include <string>
#include <vector> 

#include <GL/glut.h>

#include "Catan.hh"
#include "Sprite.hh"

using namespace std;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

const int LAND_HEX_R = 255;
const int LAND_HEX_G = 246;
const int LAND_HEX_B = 143;

//                                    hil, pas, mou, fie, for, des
const int TERRAIN_R[NUM_TERRAINS] = { 210, 152, 119, 255,   0, 245 };
const int TERRAIN_G[NUM_TERRAINS] = { 105, 251, 136, 215, 100, 245 };
const int TERRAIN_B[NUM_TERRAINS] = {  30, 152, 153,   0,   0, 220 };

const int SELECTED_HEX_R = 255;
const int SELECTED_HEX_G = 0;
const int SELECTED_HEX_B = 0;

const int SEA_R = 128;
const int SEA_G = 128;
const int SEA_B = 255;

const float HEX_HEIGHT_FACTOR  = 2.0;
const float HEX_NOTCH_FACTOR   = 0.5;          // sin(30 degs)
const float HEX_WIDTH_FACTOR   = 1.73205081;   // cos(30 degs) * 2

const int WINDOW_WIDTH         = 800; // 750
const int WINDOW_HEIGHT        = 610;

// center of upper-left tile in grid 

const int HEX_GRID_X0          = 417;  // 165
const int HEX_GRID_Y0          = 505;

// side length of one hex 

const int HEX_GRID_SIDE        = 50;

const int SETTLEMENT_WIDTH     = HEX_GRID_SIDE / 2;
const int SETTLEMENT_HEIGHT    = HEX_GRID_SIDE / 2;

// action buttons

const float BUTTON_WIDTH       = 200;
const float BUTTON_HEIGHT      = 40;
const float BUY_BUTTON_HEIGHT  = 80;  // extra space for icons to show price
const float BUTTON_SPACE       = 10;

const int BUTTON_X             = 110;
const int BUTTON_Y             = 500;

const int BUTTON_R             = 192;
const int BUTTON_G             = 192;
const int BUTTON_B             = 192;

//const int BUTTON_CHAR_WIDTH    = 12;

// where info on the game state is printed

//const int STATE_X            = WINDOW_WIDTH/2;
const int STATE_X            = BUTTON_X;
const int STATE_LINE1_Y      = WINDOW_HEIGHT - 25;
const int STATE_LINE2_Y      = WINDOW_HEIGHT - 50;
const int STATE_CHAR_WIDTH   = 12;

const int TOKEN_VAL_CHAR_WIDTH = 24;

const int TEXTURE_BRICK        = 1;
const int TEXTURE_WOOL         = 2;
const int TEXTURE_ORE          = 3;
const int TEXTURE_GRAIN        = 4;
const int TEXTURE_LUMBER       = 5;

const int TEXTURE_BOARD        = 6;

const int TEXTURE_HILLS        = 7;
const int TEXTURE_PASTURE      = 8;
const int TEXTURE_MOUNTAINS    = 9;
const int TEXTURE_FIELDS       = 10;
const int TEXTURE_FOREST       = 11;
const int TEXTURE_DESERT       = 12;

const int TEXTURE_WILDCARD     = 13;
const int TEXTURE_ROBBER       = 14;

const int CARD_WIDTH           = 43;
const int CARD_HEIGHT          = 66;
const int CARD_SPACING         = 10;
const int CARD_X_OFFSET        = 50;
const int CARD_Y               = 65;   // 75
const int RESOURCE_Y           = 20;   // 25

const int DEV_CARD_X           = 687;
const int DEV_CARD_Y           = CARD_Y;
const int DEV_CARD_W           = 190;
const int DEV_CARD_H           = CARD_HEIGHT;

const int VICTORY_PTS_X        = HEX_GRID_X0 - 150;  
const int VICTORY_PTS_Y        = CARD_Y;

const int ROAD_THICKNESS       = 8;

const float RESOURCE_SUB_S     = 0.15;
const float RESOURCE_SUB_T     = 0.26;
const float RESOURCE_SUB_SDELTA= 0.70;
const float RESOURCE_SUB_TDELTA= 0.46;

const int UNKNOWN_CURSOR_POSITION = -1;

const int NUM_WILDCARD_PORT_ICONS                  = 4;
const int WILDCARD_PORT_X[NUM_WILDCARD_PORT_ICONS] = { 245, 633, 722, 633 };
const int WILDCARD_PORT_Y[NUM_WILDCARD_PORT_ICONS] = { 361, 129, 285, 577 };

//                                           bri, woo, ore, gra, lum
const int RESOURCE_PORT_X[NUM_RESOURCES] = { 454, 721, 451, 327, 333};
const int RESOURCE_PORT_Y[NUM_RESOURCES] = { 130, 422, 577, 503, 205};

const int OFFWHITE_R  = 245;
const int OFFWHITE_G  = 245;
const int OFFWHITE_B  = 200;

//----------------------------------------------------------------------------

void read_sprites();

void draw_robber(float, float, Sprite *);
void draw_road(float, float, float, float, int, int, int);
void draw_settlement(float, float, float, float, int, int, int);
void draw_city(float, float, int, int, int);

void draw_centered_bitmap_string(float x, float y, char *text, void *, int r, int g, int b);

void draw_centered_string(float, float, float, char *, int, int, int);
void draw_string(float, float, float, char *, int = 255, int = 255, int = 255, bool = true);
void draw_line(float, float, float, float, int, int, int, int);
void draw_filled_circle(float, float, float, int, int, int);

void init_opengl(string, int, int);

void init_random();
double uniform_random(double, double);
int uniform_random_int(int, int);

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#endif
