//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"
#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// 000xx
// 0000x
// 00000
// x0000
// xx000

// is hex H good and next to this one?
// false if they are the same hex
// false if either of the hexes is not a board hex

bool Hex::is_adjacent(Hex *H)
{
  if (!H)
    return false;
  else if (row == H->row && col == H->col)
    return false;
  else if (abs(row - H->row) <= 1 && abs(col - H->col) <= 1) {
    if ((H->row == row - 1 && H->col == col + 1) || (H->row == row + 1 && H->col == col - 1))
      return false;
    else
      return true;
  }
  else
    return false;
}

//----------------------------------------------------------------------------

bool Hex::is_adjacent(Intersection *I)
{
  if (I &&
      (((I->row - row) == 0 && (I->col - 2*col) >= 0 && (I->col - 2*col) <= 2) || 
       ((I->row - row) == 1 && (I->col - 2*col) >= 1 && (I->col - 2*col) <= 3)))
    return true;
  else
    return false;
}

//----------------------------------------------------------------------------

//       0
//       ^
//      / \
//  5  /   \ 1 = I?
//     | H |
//     |   |
//  4  \   / 2
//      \ /
//       v
//       3

// which of 0-5 is intersection I, if any?  if none, return NO_INTERSECTION

int Hex::intersection_index(Intersection *I)
{  
  if (I) {
    if ((I->row - row) == 0) {        // could be 5, 0, or 1
      if ((I->col - 2*col) == 0)
	return 5;
      if ((I->col - 2*col) == 1)
	return 0;
      if ((I->col - 2*col) == 2)
	return 1;
      else 
	return NO_INTERSECTION;
    }
    else if ((I->row - row) == 1) {   // could be 4, 3, or 2
      if ((I->col - 2*col) == 1)
	return 4;
      if ((I->col - 2*col) == 2)
	return 3;
      if ((I->col - 2*col) == 3)
	return 2;
      else 
	return NO_INTERSECTION;
    }
    else
      return NO_INTERSECTION;
  }
  else
    return NO_INTERSECTION;
}

//----------------------------------------------------------------------------

// return pointer to intersection with specified index 
// (see diagram for intersection_index() function)

Intersection *Hex::find_intersection_with_index(int index)
{
  int inter_row, inter_col;

  for (inter_row = 0; inter_row < MAX_VERTICAL_INTERSECTIONS; inter_row++) 
    for (inter_col = 0; inter_col < MAX_HORIZONTAL_INTERSECTIONS; inter_col++) 
      if (intersection_index(board->intersection[inter_row][inter_col]) == index)
	return board->intersection[inter_row][inter_col];

  return NULL;
}

//----------------------------------------------------------------------------

// This hex has made a resource.  Note which intersections should get that resource

// need to see if there's a building there and who it belongs to

void Hex::distribute_resource_to_adjacent_intersections()
{
  int i;

  for (i = 0; i < NUM_INTERSECTIONS_PER_HEX; i++)
    if (inter[i]->building == PIECE_SETTLEMENT) 
      game->player[inter[i]->owned_by]->resources[terrain] += RESOURCE_INCREMENT_SETTLEMENT;
    else if (inter[i]->building == PIECE_CITY) 
      game->player[inter[i]->owned_by]->resources[terrain] += RESOURCE_INCREMENT_CITY;
}

//----------------------------------------------------------------------------

// print representation of all intersections which touch this one

void Hex::print_intersection_adjacencies()
{
  int inter_row, inter_col;

  for (inter_row = 0; inter_row < MAX_VERTICAL_INTERSECTIONS; inter_row++) {
    for (inter_col = 0; inter_col < MAX_HORIZONTAL_INTERSECTIONS; inter_col++) {
      if (is_adjacent(board->intersection[inter_row][inter_col]))
	cout << "+";
      else
	cout << ".";
    }
    cout << "\n";
  }
}

//----------------------------------------------------------------------------

// is this hex's token value 6 or 8?

bool Hex::is_special()
{
  return (token_value == SPECIAL_TOKEN_VALUE1 || token_value == SPECIAL_TOKEN_VALUE2);
}

//----------------------------------------------------------------------------

// is this hex next to a 6 or 8?

bool Hex::is_adjacent_to_special()
{
  int hex_row, hex_col;

  for (hex_row = 0; hex_row < MAX_HEXES_PER_SIDE; hex_row++) 
    for (hex_col = 0; hex_col < MAX_HEXES_PER_SIDE; hex_col++) 
      if (is_adjacent(board->hex[hex_row][hex_col]) && board->hex[hex_row][hex_col]->is_special()) 
	return true;
  
  return false;
}

//----------------------------------------------------------------------------

// print representation of all hexes which touch this one

void Hex::print_hex_adjacencies()
{
  int hex_row, hex_col;

  for (hex_row = 0; hex_row < MAX_HEXES_PER_SIDE; hex_row++) {
    for (hex_col = 0; hex_col < MAX_HEXES_PER_SIDE; hex_col++) {
      if (hex_row == row && hex_col == col)
	cout << "o";
      else if (is_adjacent(board->hex[hex_row][hex_col])) 
	cout << "+";
      else
	cout << ".";
    }
    cout << "\n";
  }
}


//----------------------------------------------------------------------------

// is 2-D point (x, y) inside the polygon defined by vertices in x_window, y_window (specified in clockwise order)?
 
// taken from Randolph Franklin's code at http://local.wasp.uwa.edu.au/~pbourke/geometry/insidepoly/

bool Hex::window_point_inside(float x, float y)
{
  int i, j;
  bool c = false;

  for (i = 0, j = NUM_INTERSECTIONS_PER_HEX - 1; i < NUM_INTERSECTIONS_PER_HEX; j = i++) {
    if ((((inter[i]->y_window <= y) && (y < inter[j]->y_window)) ||
	 ((inter[j]->y_window <= y) && (y < inter[i]->y_window))) &&
	(x < (inter[j]->x_window - inter[i]->x_window) * (y - inter[i]->y_window) / (inter[j]->y_window - inter[i]->y_window) + inter[i]->x_window))
      c = !c;
  }

  /*
  for (i = 0, j = NUM_INTERSECTIONS_PER_HEX - 1; i < NUM_INTERSECTIONS_PER_HEX; j = i++) {
    if ((((y_window[i] <= y) && (y < y_window[j])) ||
	 ((y_window[j] <= y) && (y < y_window[i]))) &&
	(x < (x_window[j] - x_window[i]) * (y - y_window[i]) / (y_window[j] - y_window[i]) + x_window[i]))
      c = !c;
  }
  */

  return c;
}

//----------------------------------------------------------------------------

// "vertical" hex (points at top and bottom, like below)

//       0
//       ^
//      / \
//  5  /   \ 1
//     |   |
//     |   |
//  4  \   / 2
//      \ /
//       v
//       3

// this is like drawing a circle with 60 degree increments between vertices
// this function should be called exactly once per hex

void Hex::compute_window_vertices(float x, float y, float side_length)
{
  int i;
  double theta, dtheta;
  Intersection *I;

  // remember center

  xc = x;
  yc = y;

  // remember vertices in clockwise order starting from 0 (see above)

  dtheta = 2 * M_PI / 6.0;

  for (theta = 0.5*M_PI, i = 0; i < NUM_INTERSECTIONS_PER_HEX; theta -= dtheta, i++) {

    I = find_intersection_with_index(i);
    inter[i] = I;

    x_window_diff[i] = side_length * cos(theta); 
    y_window_diff[i] = side_length * sin(theta); 

    if (I) {
      I->x_window = x + x_window_diff[i];
      I->y_window = y + y_window_diff[i];
    }
  }
}

//----------------------------------------------------------------------------

// draw filled

void Hex::draw_terrain()
{
  // solid color

  /*
  float r, g, b;
  int i;

  r = TERRAIN_R[terrain]; 
  g = TERRAIN_G[terrain]; 
  b = TERRAIN_B[terrain]; 

  if (terrain != TERRAIN_DESERT && terrain != TERRAIN_FOREST) {

    // change pen color
    
    glColor3ub(r, g, b);
    
    glBegin(GL_POLYGON);
    
    for (i = 0; i < NUM_INTERSECTIONS_PER_HEX; i++) 
      glVertex2f(inter[i]->x_window, inter[i]->y_window);
    
    glEnd();
    
    // restore pen color
    
    glColor3f(1,1,1);
  }

  */

  // texture

  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);
  
  game->terrain_sprite[terrain]->draw(xc, yc, 
				      game->terrain_sprite[terrain]->imwidth, game->terrain_sprite[terrain]->imheight,
				      0, 1, 1, 1);
  
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);

}

//----------------------------------------------------------------------------

void Hex::draw_outline(int r, int g, int b, int thickness, float inset_fraction)
{
  int i;

  // change pen color

  glColor3ub(r, g, b);
  glLineWidth(thickness);

  glBegin(GL_LINE_LOOP);

  for (i = 0; i < NUM_INTERSECTIONS_PER_HEX; i++) 
    //    glVertex2f(inter[i]->x_window, inter[i]->y_window);
    glVertex2f(xc + inset_fraction * x_window_diff[i], yc + inset_fraction * y_window_diff[i]);

  glEnd();

  // restore pen color

  glColor3f(1,1,1);
  glLineWidth(1);
}

//----------------------------------------------------------------------------

void Hex::draw_token_value()
{
  float r, g, b;
  int i;

  //  draw_filled_circle(xc, yc, 20, 255, 255, 255);
  //  draw_filled_circle(xc, yc, 18, 250, 250, 250);
  draw_filled_circle(xc, yc, 18, OFFWHITE_R, OFFWHITE_G, OFFWHITE_B);

  char *value_string = (char *) malloc(MAX_STRLEN*sizeof(char));

  //  if (token_value != NO_TOKEN) {
    sprintf(value_string, "%i", token_value);

    if (is_special())
      //  draw_centered_string(xc, yc, TOKEN_VAL_CHAR_WIDTH, value_string, 192, 0, 0);
      //      draw_centered_bitmap_string(xc, yc, value_string, GLUT_BITMAP_TIMES_ROMAN_24, 192, 0, 0);
      draw_centered_bitmap_string(xc, yc, value_string, GLUT_BITMAP_TIMES_ROMAN_24, 255, 0, 0);
    else
      //      draw_centered_string(xc, yc, TOKEN_VAL_CHAR_WIDTH, value_string, 0, 0, 0);
      draw_centered_bitmap_string(xc, yc, value_string, GLUT_BITMAP_TIMES_ROMAN_24, 0, 0, 0);

    //  }

  free(value_string);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

