//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"
#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Intersection::Intersection(int r, int c, Board *B)
{
  row = r; 
  col = c; 
  building = PIECE_NONE; 
  board = B; 

  int i;

  // default: not a port

  port = RESOURCE_NOTHING;

  // is this a wildcard port?

  for (i = 0; i < NUM_WILDCARD_PORTS; i++)
    if (r == WILDCARD_PORT_ROW[i] && c == WILDCARD_PORT_COL[i]) {
      port = RESOURCE_WILDCARD;
      break;
    }

  // is this a brick port?

  for (i = 0; i < NUM_RESOURCE_PORTS; i++)
    if (r == BRICK_PORT_ROW[i] && c == BRICK_PORT_COL[i]) {
      port = RESOURCE_BRICK;
      break;
    }

  // is this a wool port?

  for (i = 0; i < NUM_RESOURCE_PORTS; i++)
    if (r == WOOL_PORT_ROW[i] && c == WOOL_PORT_COL[i]) {
      port = RESOURCE_WOOL;
      break;
    }

  // is this a ore port?

  for (i = 0; i < NUM_RESOURCE_PORTS; i++)
    if (r == ORE_PORT_ROW[i] && c == ORE_PORT_COL[i]) {
      port = RESOURCE_ORE;
      break;
    }

  // is this a grain port?

  for (i = 0; i < NUM_RESOURCE_PORTS; i++)
    if (r == GRAIN_PORT_ROW[i] && c == GRAIN_PORT_COL[i]) {
      port = RESOURCE_GRAIN;
      break;
    }

  // is this a lumber port?

  for (i = 0; i < NUM_RESOURCE_PORTS; i++)
    if (r == LUMBER_PORT_ROW[i] && c == LUMBER_PORT_COL[i]) {
      port = RESOURCE_LUMBER;
      break;
    }  
}

//----------------------------------------------------------------------------

void Intersection::draw()
{
  if (building == PIECE_SETTLEMENT)
    draw_settlement(x_window, y_window, SETTLEMENT_WIDTH, SETTLEMENT_HEIGHT, 
		    PLAYER_R[owned_by], PLAYER_G[owned_by], PLAYER_B[owned_by]);
  else if (building == PIECE_CITY)
    draw_city(x_window, y_window, PLAYER_R[owned_by], PLAYER_G[owned_by], PLAYER_B[owned_by]);
}

//----------------------------------------------------------------------------

// seg faulting here now that i'm checking for adjacent settlements/cities

bool Intersection::is_adjacent(Intersection *I)
{
  if (!I)
    return false;

  if ((I->row == row && abs(I->col - col) == 1) ||                 // I level with this intersection 
      (I->row == row + 1 && !(col % 2) && I->col == col + 1) ||    // if I below it, col must be even
      (I->row == row - 1 && (col % 2) && I->col == col - 1))       // if I above it, col must be odd
    return true;
  else 
    return false;
}

//----------------------------------------------------------------------------

void Intersection::draw_highlighted(int r, int g, int b, int thickness)
{
  glColor3f(r, g, b);
  glPointSize(thickness);
  
  glBegin(GL_POINTS);
  
  glVertex2f(x_window, y_window);
  
  glEnd();
  
  // restore pen state
  
  glColor3f(1,1,1);
  glPointSize(1);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

