//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan-specific stuff
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Player::Player(int player_num)
{
  int i;

  player_number = player_num;

  for (i = 0; i < NUM_RESOURCES; i++) {
    resources[i] = 0;
    trade_ratio[i] = TRADE_RATIO_ANYTIME;
  }

  current_card = CARD_NONE;

  num_settlements = 0;
  num_cities = 0;

  num_roads = 0;
  longest_road = false;

  num_knights = 0;
  largest_army = false;

  dev_card_victory_points = 0;

}

//----------------------------------------------------------------------------

// check existing ratios; if ratios for any matching resources are higher than new ratio, change them to it

// should not be called with RESOURCE_NOTHING

void Player::reset_trade_ratios(int resource_type, int new_ratio)
{
  int i;

  // anything matches

  if (resource_type == RESOURCE_WILDCARD) {
    for (i = 0; i < NUM_RESOURCES; i++)
      if (trade_ratio[i] > new_ratio)
	trade_ratio[i] = new_ratio;
  }

  // specific

  else {
    if (trade_ratio[resource_type] > new_ratio)
      trade_ratio[resource_type] = new_ratio;
  }
}

//----------------------------------------------------------------------------

void Player::adjust_trade_ratios_if_port(int port_type)
{
  int i;

  if (port_type == RESOURCE_NOTHING)
    return;

  else if (port_type == RESOURCE_WILDCARD) {
    reset_trade_ratios(RESOURCE_WILDCARD, TRADE_RATIO_WILDCARD_PORT);
    return;
  }
  
  // some kind of specific resource port

  else 
    reset_trade_ratios(port_type, TRADE_RATIO_RESOURCE_PORT);
}

//----------------------------------------------------------------------------

void Player::compute_victory_points()
{
  victory_points = 
    POINTS_SETTLEMENT * num_settlements +
    POINTS_CITY * num_cities +
    POINTS_LONGEST_ROAD * longest_road + 
    POINTS_LARGEST_ARMY * largest_army +
    dev_card_victory_points;
}

//----------------------------------------------------------------------------

void Player::print()
{
  int i;

  cout << "You have...\n";

  for (i = 0; i < NUM_RESOURCES; i++)
    cout << RESOURCE_CODE[i] << ":\t" << resources[i] << "\n";
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
