//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Catan knock-off
//
// by Christopher Rasmussen, cer@cis.udel.edu
//
// copyright April, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "Catan.hh"
#include "Catan_Draw.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// globals

Catan *Catan_Game;
int road_constraint = EXTENDS_ROAD;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// has a key been released?

void keyboardup(unsigned char key, int x, int y)
{
  // quit

  if (key == 'q')
    exit(1);

  // drop piece if there is one in hand
  // resources paid are not restored :(

  else if (key == 'd') 
    Catan_Game->end_piece_placement_mode();

  // "force" road to be dropped whereever player wants

  else if (key == 'r') {
    if (road_constraint == EXTENDS_ROAD) {
      road_constraint = NO_CONSTRAINT;
      cout << "no road constraint\n";
    }
    else {
      road_constraint = EXTENDS_ROAD;
      cout << "must extend road\n";
    }
  }

  // cheat code to add resources to current player's account

  else if (key == 'p' && Catan_Game->phase == TURN_PHASE) {
    for (int i = 0; i < NUM_RESOURCES; i++)
      Catan_Game->player[Catan_Game->which_player]->resources[i]++;
    Catan_Game->update_resource_sensitive_buttons();

  }

  // redraw now

  glutPostRedisplay();
}

//----------------------------------------------------------------------------

// has a mouse click event occurred?

void mouse(int button, int state, int x, int y)
{
  Intersection *I;
  Edge *E;

  // convert to bottom left origin

  y = WINDOW_HEIGHT - y;

  if (button == GLUT_LEFT_BUTTON) {
    
    if (state == GLUT_UP) {

      //      printf("%i %i\n", x, y);

      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      if (Catan_Game->phase == SETUP_PHASE) {

	// settlement in hand?

	if (Catan_Game->current_piece == PIECE_SETTLEMENT) {
	  if (!Catan_Game->try_to_place_building(Catan_Game->current_piece, x, y, false))
	    return;
	}
	
	// road in hand

	else {   

	  if (!Catan_Game->try_to_place_road(x, y, TOUCHES_ONLY_SETTLEMENT))
	    return;

	  // move to next player after road placement

	  Catan_Game->next_player();
	}
      }

      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      else if (Catan_Game->phase == TURN_PHASE) {

	// try to drop robber at click site

	if (Catan_Game->current_piece == PIECE_ROBBER) {
	  if (!Catan_Game->try_to_place_robber(x, y, true))
	    return;
	}

	// try to drop building at click site

	else if (Catan_Game->current_piece == PIECE_SETTLEMENT || Catan_Game->current_piece == PIECE_CITY) {
	  if (!Catan_Game->try_to_place_building(Catan_Game->current_piece, x, y, true))
	    return;
	}

	// try to drop road at click site

	else if (Catan_Game->current_piece == PIECE_ROAD) {
	  if (!Catan_Game->try_to_place_road(x, y, road_constraint))
	    return;
	}

	// else no piece in hand, so handle button clicks in ordinary way...

	// roll dice

	else if (Catan_Game->roll_dice_button->is_clicked(x, y)) {
	  Catan_Game->clear_highlights(); 
	  Catan_Game->distribute_production(Catan_Game->roll_dice());
	  Catan_Game->update_resource_sensitive_buttons();
	  Catan_Game->roll_dice_button->set_inactive();
	  Catan_Game->end_turn_button->set_active();
	}

	// trade resources

	else if (Catan_Game->trade_button->is_clicked(x, y)) {

	  // set resources you can trade from active and everything else inactive

	  Catan_Game->deactivate_action_buttons();
	  Catan_Game->set_resource_button_activations();

	}

	// buy and put road piece in hand

	else if (Catan_Game->buy_road_button->is_clicked(x, y)) {
	  Catan_Game->execute_player_purchase(COST_ROAD);
	  Catan_Game->start_piece_placement_mode(PIECE_ROAD);
	}

	// buy and put settlement piece in hand 

	else if (Catan_Game->buy_settlement_button->is_clicked(x, y)) {
	  Catan_Game->execute_player_purchase(COST_SETTLEMENT);
	  Catan_Game->start_piece_placement_mode(PIECE_SETTLEMENT);
	}

	// buy and put city piece in hand

	else if (Catan_Game->buy_city_button->is_clicked(x, y)) {
	  Catan_Game->execute_player_purchase(COST_CITY);
	  Catan_Game->start_piece_placement_mode(PIECE_CITY);
	}

	// buy development card, deal it, show it, and apply its effect immediately

	else if (Catan_Game->buy_development_card_button->is_clicked(x, y)) {
	  Catan_Game->execute_player_purchase(COST_DEVELOPMENT_CARD);
	  Catan_Game->deal_and_play_development_card();	
	}

	// move to next player

	else if (Catan_Game->end_turn_button->is_clicked(x, y)) 
	  Catan_Game->next_player();

	// resource card clicked?  this is where trade giving/taking is handled

	else 
	  Catan_Game->handle_resource_buttons(x, y);
      }

      glutPostRedisplay();    
    }
  }
}

//----------------------------------------------------------------------------

// mouse moved with NO button down

void passivemotion(int x, int y)
{
  Catan_Game->cursor_x = x;
  Catan_Game->cursor_y = WINDOW_HEIGHT - y;

  // redraw
  
  glutPostRedisplay();
}

//----------------------------------------------------------------------------

void display() 
{
  // wipe background (don't really need this if drawing a background image)

  glClear(GL_COLOR_BUFFER_BIT);

  // draw stuff

  if (Catan_Game) {

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);

    Catan_Game->board_background_sprite->draw(504, 354, 
					      Catan_Game->board_background_sprite->imwidth, Catan_Game->board_background_sprite->imheight, 
					      0, 1, 1, 1);
 
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    Catan_Game->draw_state();                // text at the top that tells us whose turn it is, etc.
    Catan_Game->board->draw();               // the board itself
    Catan_Game->draw_player_resources();     // stuff at the bottom that shows how many resource cards current player has
    Catan_Game->draw_player_victory_points();
    Catan_Game->draw_action_buttons();       // how the player chooses what to do
    Catan_Game->draw_cursor();
  }

  // finish up

  glutSwapBuffers();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

int main(int argc, char** argv)
{
  // boilerplate

  glutInit(&argc, argv);
  init_opengl("Catan", WINDOW_WIDTH, WINDOW_HEIGHT); 
  init_random();

  // register callback routines

  glutDisplayFunc(display);
  glutKeyboardUpFunc(keyboardup);
  glutPassiveMotionFunc(passivemotion);
  glutMouseFunc(mouse);

 // initialize game

  int num_players;
  
  if (argc > 1) {
    num_players = atoi(argv[1]);
    if (num_players < 2 || num_players > 4) {
      cout << "This game is for 2-4 players\n";
      exit(1);
    }
  }
  else
    num_players = 2;   
  Catan_Game = new Catan(num_players);
  finalize(Catan_Game);
  
  // enter event-driven loop

  glutMainLoop(); 		    

  // this will never be reached

  return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
